
/* Generated by Interface Builder */

#import "SpectrumView.h"

#import <Cocoa/Cocoa.h>
#import <math.h>

float back_ground = 0.0/3.0, draw_gray = 3.0/3.0, marker_gray = 1.0/3.0;
float sines[4096],cosines[4096];
int last_length = 0;

@implementation SpectrumView

- setBackGround: (float) gray
{
    back_ground = gray;
    return self;
}

- setDraw: (float) gray
{
    draw_gray = gray;
    return self;
}

- displayData: (int) powerOfFour array: (float *) f
{
    int temp;
    temp = pow(4.0,(double) powerOfFour);
    [self fhtRX4: powerOfFour array: f];
    [self logMag: temp array: f floor: -60];
    [self drawSpectrum: temp / 2 array: f];
    return self;
}


- drawSpectrum: (int) len array: (float *) far
{
    [self drawSpectrum: len array:far erase: FALSE];
    return self;
}

- drawSpectrum: (int) length array: (float *) far erase: (BOOL) er
{

	int i,incr;
	double xstep,ymax,max=0.0;
	NSRect frame = [self frame];	
	NSBezierPath *newPath = [NSBezierPath bezierPath];

	if(path != newPath)
	{
		[path release];
		path = [newPath copyWithZone:[self zone]];
	}
	
	xstep = frame.size.width / length;
	ymax = frame.size.height;
	incr = length / frame.size.width;
	
	[path setLineWidth:2];
	[path moveToPoint: (NSPoint) {0.0, far[0] * ymax}];
	for (i=1;i<length;i++)
	{
		if (far[i]>max) max = far[i];
		if (incr==0)
		{
			[path lineToPoint: (NSPoint) {(double) i * xstep, max * ymax}];
			max = 0.0;	
		}
		else if ((i%incr)==0)
		{
			[path lineToPoint: (NSPoint) {(double) i * xstep, max * ymax}];
			max = 0.0;
		}
	}

	[self setNeedsDisplay:YES];
	return self;
}

- placeMarkerAt: (float) pos height: (float) height erase: (BOOL) erase;
{
/*    double xpos,ypos;
		NSRect frame = [self frame];
    xpos = frame.size.width * pos;
    ypos = frame.size.height * height;
    [self lockFocus];
    //[self setOpaque: TRUE];
		//[path setLineWidt:0.1];
    //if (erase) PSsetgray(back_ground);
    //else PSsetgray(marker_gray);
    if (xpos==frame.size.width) xpos -= 1.0;
    //PSmoveto(xpos,0.0);
    //PSlineto(xpos,ypos);
    //PSstroke();
    //PSflushgraphics();
    [self unlockFocus];*/
    return self;
}

- placeHorizontal: (float) height;
{
/*
    double xpos,ypos;
		NSRect frame = [self frame];
    xpos = frame.size.width;
    ypos = frame.size.height * height;
    [self lockFocus];
    //[self setOpaque: TRUE];
    if (ypos<1.0) ypos = 1.0;
    [path setLineWidt:0.1];
    //PSsetgray(marker_gray);
    //PSmoveto(0.0,ypos);
    //PSlineto(xpos,ypos);
    //PSstroke();
    //PSflushgraphics();
    [self unlockFocus];*/
    return self;
}
    

- (void)drawRect:(NSRect) rect;
{
		NSGraphicsContext *currentContext = [NSGraphicsContext currentContext];
		
		[[NSColor blackColor] set];
    [NSGraphicsContext saveGraphicsState];
    [path stroke];
		
    [currentContext restoreGraphicsState];

}


- logMag: (int) size array: (float *) f floor: (float) fl

/* calculates the log magnitude of the data, using the Hartley
	identity |X(f)| = sqrt(X(f)^2 + X(N-f)^2) * scale factor
	I ignore the scale factor because the data gets normalized.	*/

{
    int i;
    float t1=0.0,t=0.0,t2=-12;
    t2 = fl / 5.0;
    f[0] *= 2*f[0];
    if (f[0] > t1)
        t1 = f[0];
    for (i=1;i<size/2;i++)	{
    	t = f[i]*f[i] + f[size-i]*f[size-i];
	f[i] = t;
	if (t>t1)
	   t1 = t;
    }
    for (i=0;i<size/2;i++)	{
    	t = log(f[i]/t1);
	if (t<t2)
	   t = t2;
	f[i] = 1 - t/t2;
    }
    return self;
}

/*********************   FHT ROUTINES  **************************/
    #define PI 3.141592654782
    #define SQRT_TWO 1.414213562
    #define TWO_PI 6.283185309564

void make_sines(int power_of_four)
{
    int i,length;
    float freq,temp;
    if (power_of_four > 6)	{
        printf("You're calling FHTs bigger than 4096!!\n");
	printf("Edit SpectrumView and increase the size\n");
	exit(0);
    }
    length = pow(4.0, (double) power_of_four);
    if (length!=last_length)	{
        last_length = length;
        freq = 2.0 * PI / length;
        for (i=0;i<length;i++) {
            temp = freq * i;
	    sines[i] = sin(temp);
	    cosines[i] = cos(temp);
        }
    }
    return;
}

- fhtRX4: (int) powerOfFour array: (float *) array
{
    /* In place Fast Hartley Transform of floating point data in array.
    	Size of data array must be power of four. Lots of sets of four 
	inline code statements, so it is verbose and repetative but fast. 
	The Fast Hartley Transform algorithm is patented, and is documented
	in "The Hartley Transform", by Ronald N. Bracewell.
	This routine was converted to C from a BASIC routine in the above book,
	that routine Copyright 1985, The Board of Trustees of Stanford University */
	
    register int j=0,i=0,k=0,L=0;
    int n=0,n4=0,d1=0,d2=0,d3=0,d4=0,d5=1,d6=0,d7=0,d8=0,d9=0;
    int L1=0,L2=0,L3=0,L4=0,L5=0,L6=0,L7=0,L8=0;
    int n_over_d3;
    float r=0.0;
    int a1=0,a2=0,a3=0;
    float s1,s2,s3,c1,c2,c3;
    float t=0.0,t1=0.0,t2 =0.0,t3=0.0,t4=0.0,t5=0.0,t6=0.0,t7=0.0,t8=0.0;
    float t9=0.0,t0=0.0;
    if (powerOfFour!=last_length)  make_sines(powerOfFour);
    n = pow(4.0 , (double) powerOfFour);
    n4 = n / 4;
    r = SQRT_TWO;						// use SQRT_TWO directly
    j = 1;
    i = 0;
    while (i<n-1)	{
    	i++;
	if (i<j)	{
    		t = array[j-1];
		array[j-1] = array[i-1];
		array[i-1] = t;
    	}
    	k = n4;
    	while ((3*k)<j)	{
    		j -= 3 * k;
		k /= 4;
    	}
    	j += k;
    }
    for (i=0;i<n;i += 4) {
    	t5 = array[i];
	t6 = array[i+1];
    	t7 = array[i+2];
	t8 = array[i+3];
	t1 = t5 + t6;
    	t2 = t5 - t6;
    	t3 = t7 + t8;
    	t4 = t7 - t8;
	array[i] = t1 + t3;
	array[i+1] = t1 - t3;
	array[i+2] = t2 + t4;
	array[i+3] = t2 - t4;
    }
    for (L=2;L<=powerOfFour;L++)  {
	d1 = pow(2.0 , L+L-3.0);
	d2=d1+d1;
	d3=d2+d2;
	n_over_d3 = n / 2 / d3;
	d4=d2+d3;
	d5=d3+d3;
	for (j=0;j<n;j += d5)	  {
	     t5 = array[j];
	     t6 = array[j+d2];
	     t7 = array[j+d3];
	     t8 = array[j+d4];
	     t1 = t5+t6;
	     t2 = t5-t6;
	     t3 = t7+t8;
	     t4 = t7-t8;
	     array[j] = t1 + t3;
	     array[j+d2] = t1 - t3;
	     array[j+d3] = t2 + t4;
	     array[j+d4] = t2 - t4;
	     d6 = j+d1;
	     d7 = j+d1+d2;
	     d8 = j+d1+d3;
	     d9 = j+d1+d4;
	     t1 = array[d6];
	     t2 = array[d7] * r;
	     t3 = array[d8];
	     t4 = array[d9] * r;
	     array[d6] = t1 + t2 + t3;
	     array[d7] = t1 - t3 + t4;
	     array[d8] = t1 - t2 + t3;
	     array[d9] = t1 - t3 - t4;
	     for (k=1;k<d1;k++)	{
		  L1 = j + k;
		  L2 = L1 + d2;
		  L3 = L1 + d3;
		  L4 = L1 + d4;
		  L5 = j + d2 - k;
		  L6 = L5 + d2;
		  L7 = L5 + d3;
		  L8 = L5 + d4;
		  a1 = (int) (k * n_over_d3) % n;
		  a2 = (a1 + a1) % n;
		  a3 = (a1 + a2) % n;
		  s1 = sines[a1];
		  s2 = sines[a2];
		  s3 = sines[a3];
		  c1 = cosines[a1];
		  c2 = cosines[a2];
		  c3 = cosines[a3];
		  t5 = array[L2] * c1 + array[L6] * s1;
		  t6 = array[L3] * c2 + array[L7] * s2;
		  t7 = array[L4] * c3 + array[L8] * s3;
		  t8 = array[L6] * c1 - array[L2] * s1;
		  t9 = array[L7] * c2 - array[L3] * s2;
		  t0 = array[L8] * c3 - array[L4] * s3;
		  t1 = array[L5] - t9;
		  t2 = array[L5] + t9;
		  t3 = - t8 - t0;
		  t4 = t5 - t7;
		  array[L5] = t1 + t4;
		  array[L6] = t2 + t3;
		  array[L7] = t1 - t4;
		  array[L8] = t2 - t3;
		  t1 = array[L1] + t6;
		  t2 = array[L1] - t6;
		  t3 = t8 - t0;
		  t4 = t5 + t7;
		  array[L1] = t1 + t4;
		  array[L2] = t2 + t3;
		  array[L3] = t1 - t4;
		  array[L4] = t2 - t3;
	     }
	}
    }		  
    return self;
}

@end
