/***********************************************************/
/*     Wave I/O soundfile access routines                  */
/*      EDIT THIS FILE ONCE FOR YOUR ARCHITECTURE          */
/*      #define one of                                     */
/*              LITTLENDIAN (intel, Windows or Linux)      */
/*           or BIGENDIAN (some suns, NeXT, SGI, other)    */
/*      then compile the .c files and enjoy                */
/*                                                         */
/*      LITTLENDIAN will cause the programs to consume     */
/*          and yield .wav files                           */
/*      BIGENDIAN will cause the programs to consume       */
/*          and yield .snd files                           */
/*                                                         */
/*      (c) Perry R. Cook, 2002                            */
/***********************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#define _EXTENSION_ .snd
#define _EXTENSION_STRING_ ".snd"

/********  NeXT/Sun BigEndian Soundfile Versions   *******/

struct soundhdr {
  char pref[4];
  long hdr_length;
  long dlength;         /* Data Length in bytes */
  long mode;
  long srate;
  long num_chans;
  char comment[20];
};

FILE *opensoundin(char *filename,struct soundhdr *hdr)    {
    FILE *myfile;
    /*   check file name for .wav or .snd extension */
    if (filename[strlen(filename)-1] == 'd')  {
        myfile = fopen(filename,"rb");
        if (!myfile) {
            printf("Open Input File Error!!\n");
            myfile = 0;
        }
        else {
            fread(hdr,8,1,myfile);
	    fread(&hdr->dlength,1,hdr->hdr_length-8,myfile);
        }
    }
    else {
        printf("You must specify a valid .snd file name\n");
        myfile = 0;
    }
    return myfile;
}

FILE *opensoundout(char *filename, struct soundhdr *hdr)    {
    FILE *myfile;
    /*   check file name for .snd extension */
    if (filename[strlen(filename)-1] == 'd')   {
        myfile = fopen(filename,"wb");
        if (!myfile) {
            printf("Open Output File Error!!\n");
            myfile = 0;
        }
        else     {
            fwrite(hdr,1,hdr->hdr_length,myfile);
        }
    }
    else {
        printf("You must specify a valid .snd file name\n");
        myfile = 0;
    }
    return myfile;
}

void fillheader(struct soundhdr *hdr, long srate)      {
    struct soundhdr myhdr = {".sn",44,0,3,44100,1,"Real Sound Synth. "};
    myhdr.srate = srate;
    myhdr.pref[3] = 'd';
    *hdr = myhdr;
}

void closesoundout(FILE *file,long totalCount)      {
    long bytes;

    fseek(file,8,SEEK_SET);
    bytes = totalCount * 2;
    fwrite(&bytes,4,1,file);
    fclose(file);
}

void closesoundin(FILE *file)      {
    fclose(file);
}

